@@ -7,6 +7,7 @@
 #include "lspclientserver.h"
 
 #include "lspclient_debug.h"
+#include "hostprocess.h"
 
 #include <QCoreApplication>
 #include <QFileInfo>
@@ -32,6 +33,7 @@ static const QString MEMBER_VERSION = QStringLiteral("version");
 static const QString MEMBER_START = QStringLiteral("start");
 static const QString MEMBER_END = QStringLiteral("end");
 static const QString MEMBER_POSITION = QStringLiteral("position");
+static const QString MEMBER_POSITIONS = QStringLiteral("positions");
 static const QString MEMBER_LOCATION = QStringLiteral("location");
 static const QString MEMBER_RANGE = QStringLiteral("range");
 static const QString MEMBER_LINE = QStringLiteral("line");
@@ -117,6 +119,15 @@ static QJsonArray to_json(const QList<LSPTextDocumentContentChangeEvent> &change
     return result;
 }
 
+static QJsonArray to_json(const QVector<LSPPosition> &positions)
+{
+    QJsonArray result;
+    for (const auto &position : positions) {
+        result.push_back(to_json(position));
+    }
+    return result;
+}
+
 static QJsonObject versionedTextDocumentIdentifier(const QUrl &document, int version = -1)
 {
     QJsonObject map{{MEMBER_URI, document.toString()}};
@@ -151,6 +162,13 @@ static QJsonObject textDocumentPositionParams(const QUrl &document, LSPPosition
     return params;
 }
 
+static QJsonObject textDocumentPositionsParams(const QUrl &document, const QVector<LSPPosition> &positions)
+{
+    auto params = textDocumentParams(document);
+    params[MEMBER_POSITIONS] = to_json(positions);
+    return params;
+}
+
 static QJsonObject referenceParams(const QUrl &document, LSPPosition pos, bool decl)
 {
     auto params = textDocumentPositionParams(document, pos);
@@ -372,6 +390,7 @@ static void from_json(LSPServerCapabilities &caps, const QJsonObject &json)
     from_json(caps.semanticTokenProvider, json.value(QStringLiteral("semanticTokensProvider")).toObject());
     auto workspace = json.value(QStringLiteral("workspace")).toObject();
     from_json(caps.workspaceFolders, workspace.value(QStringLiteral("workspaceFolders")));
+    caps.selectionRangeProvider = toBoolOrObject(json.value(QStringLiteral("selectionRangeProvider")));
 }
 
 // follow suit; as performed in kate docmanager
@@ -450,6 +469,37 @@ static LSPRange parseRange(const QJsonObject &range)
     return {startpos, endpos};
 }
 
+static std::shared_ptr<LSPSelectionRange> parseSelectionRange(QJsonValueRef selectionRange)
+{
+    auto current = std::make_shared<LSPSelectionRange>(LSPSelectionRange{});
+    std::shared_ptr<LSPSelectionRange> ret = current;
+    QJsonValue selRange = std::move(selectionRange);
+
+    while (selRange.isObject()) {
+        current->range = parseRange(selRange[MEMBER_RANGE].toObject());
+        if (!selRange[QStringLiteral("parent")].isObject()) {
+            current->parent = nullptr;
+            break;
+        }
+        selRange = selRange[QStringLiteral("parent")].toObject();
+        current->parent = std::make_shared<LSPSelectionRange>(LSPSelectionRange{});
+        current = current->parent;
+    }
+
+    return ret;
+}
+
+static QList<std::shared_ptr<LSPSelectionRange>> parseSelectionRanges(const QJsonValue &result)
+{
+    QList<std::shared_ptr<LSPSelectionRange>> ret;
+    auto selectionRanges = result.toArray();
+    for (QJsonValueRef selectionRange : selectionRanges) {
+        ret.push_back(parseSelectionRange(selectionRange));
+    }
+
+    return ret;
+}
+
 static LSPLocation parseLocation(const QJsonObject &loc)
 {
     auto uri = normalizeUrl(QUrl(loc.value(MEMBER_URI).toString()));
@@ -717,6 +767,15 @@ static QString parseClangdSwitchSourceHeader(const QJsonValue &result)
     return result.toString();
 }
 
+static LSPExpandedMacro parseExpandedMacro(const QJsonValue &result)
+{
+    LSPExpandedMacro ret;
+    const QJsonObject ob = result.toObject();
+    ret.name = ob.value(QStringLiteral("name")).toString();
+    ret.expansion = ob.value(QStringLiteral("expansion")).toString();
+    return ret;
+}
+
 static LSPTextDocumentEdit parseTextDocumentEdit(const QJsonValue &result)
 {
     LSPTextDocumentEdit ret;
@@ -977,6 +1036,9 @@ class LSPClientServer::LSPClientServerPrivate
     static constexpr int MAX_REQUESTS = 5;
     QVector<int> m_requests{MAX_REQUESTS + 1};
 
+    // currently accumulated stderr output, used to output to the message view on line level
+    QString m_currentStderrOutput;
+
 public:
     LSPClientServerPrivate(LSPClientServer *_q,
                            const QStringList &server,
@@ -992,7 +1054,8 @@ public:
         , m_folders(folders)
     {
         // setup async reading
-        QObject::connect(&m_sproc, &QProcess::readyRead, utils::mem_fun(&self_type::read, this));
+        QObject::connect(&m_sproc, &QProcess::readyReadStandardOutput, utils::mem_fun(&self_type::readStandardOutput, this));
+        QObject::connect(&m_sproc, &QProcess::readyReadStandardError, utils::mem_fun(&self_type::readStandardError, this));
         QObject::connect(&m_sproc, &QProcess::stateChanged, utils::mem_fun(&self_type::onStateChanged, this));
     }
 
@@ -1089,7 +1152,7 @@ private:
         return RequestHandle();
     }
 
-    void read()
+    void readStandardOutput()
     {
         // accumulate in buffer
         m_receive.append(m_sproc.readAllStandardOutput());
@@ -1192,6 +1255,26 @@ private:
         }
     }
 
+    void readStandardError()
+    {
+        // append new stuff to our buffer, we assume UTF-8 output
+        m_currentStderrOutput.append(QString::fromUtf8(m_sproc.readAllStandardError()));
+
+        // now, cut out all full lines
+        LSPShowMessageParams msg;
+        if (const int lastNewLineIndex = m_currentStderrOutput.lastIndexOf(QLatin1Char('\n')); lastNewLineIndex >= 0) {
+            msg.message = m_currentStderrOutput.left(lastNewLineIndex);
+            m_currentStderrOutput.remove(0, lastNewLineIndex + 1);
+        }
+
+        // emit the output lines if non-empty
+        // this might strip empty lines in error output, but that is better then a spammed output view
+        if (!msg.message.isEmpty()) {
+            msg.type = LSPMessageType::Log;
+            Q_EMIT q->logMessage(msg);
+        }
+    }
+
     static QJsonObject init_error(const LSPErrorCode code, const QString &msg)
     {
         return QJsonObject{{MEMBER_ERROR, QJsonObject{{MEMBER_CODE, static_cast<int>(code)}, {MEMBER_MESSAGE, msg}}}};
@@ -1266,6 +1349,7 @@ private:
                                             {QStringLiteral("codeAction"), codeAction},
                                             {QStringLiteral("semanticTokens"), semanticTokens},
                                             {QStringLiteral("synchronization"), QJsonObject{{QStringLiteral("didSave"), true}}},
+                                            {QStringLiteral("selectionRange"), QJsonObject{{QStringLiteral("dynamicRegistration"), false}}},
                                         },
                                   },
                                   {QStringLiteral("window"),
@@ -1301,7 +1385,7 @@ private:
     }
 
 public:
-    bool start()
+    bool start(bool forwardStdError)
     {
         if (m_state != State::None) {
             return true;
@@ -1315,10 +1399,10 @@ public:
         // start LSP server in project root
         m_sproc.setWorkingDirectory(m_root.toLocalFile());
 
-        // at least we see some errors somewhere then
-        m_sproc.setProcessChannelMode(QProcess::ForwardedErrorChannel);
+        // we handle stdout/stderr internally, important stuff via stdout
+        m_sproc.setProcessChannelMode(forwardStdError ? QProcess::ForwardedErrorChannel : QProcess::SeparateChannels);
         m_sproc.setReadChannel(QProcess::QProcess::StandardOutput);
-        m_sproc.start(program, args);
+        startHostProcess(m_sproc, program, args);
         const bool result = m_sproc.waitForStarted();
         if (result) {
             setState(State::Started);
@@ -1401,6 +1485,12 @@ public:
         return send(init_request(QStringLiteral("textDocument/signatureHelp"), params), h);
     }
 
+    RequestHandle selectionRange(const QUrl &document, const QVector<LSPPosition> &positions, const GenericReplyHandler &h)
+    {
+        auto params = textDocumentPositionsParams(document, positions);
+        return send(init_request(QStringLiteral("textDocument/selectionRange"), params), h);
+    }
+
     RequestHandle clangdSwitchSourceHeader(const QUrl &document, const GenericReplyHandler &h)
     {
         auto params = QJsonObject{{MEMBER_URI, document.toString()}};
@@ -1412,6 +1502,12 @@ public:
         return send(init_request(QStringLiteral("$/memoryUsage"), QJsonObject()), h);
     }
 
+    RequestHandle rustAnalyzerExpandMacro(const QUrl &document, const LSPPosition &pos, const GenericReplyHandler &h)
+    {
+        auto params = textDocumentPositionParams(document, pos);
+        return send(init_request(QStringLiteral("rust-analyzer/expandMacro"), params), h);
+    }
+
     RequestHandle documentFormatting(const QUrl &document, const LSPFormattingOptions &options, const GenericReplyHandler &h)
     {
         auto params = documentRangeFormattingParams(document, nullptr, options);
@@ -1649,9 +1745,9 @@ const LSPServerCapabilities &LSPClientServer::capabilities() const
     return d->capabilities();
 }
 
-bool LSPClientServer::start()
+bool LSPClientServer::start(bool forwardStdError)
 {
-    return d->start();
+    return d->start(forwardStdError);
 }
 
 void LSPClientServer::stop(int to_t, int to_k)
@@ -1724,6 +1820,12 @@ LSPClientServer::signatureHelp(const QUrl &document, const LSPPosition &pos, con
     return d->signatureHelp(document, pos, make_handler(h, context, parseSignatureHelp));
 }
 
+LSPClientServer::RequestHandle
+LSPClientServer::selectionRange(const QUrl &document, const QVector<LSPPosition> &positions, const QObject *context, const SelectionRangeReplyHandler &h)
+{
+    return d->selectionRange(document, positions, make_handler(h, context, parseSelectionRanges));
+}
+
 LSPClientServer::RequestHandle LSPClientServer::clangdSwitchSourceHeader(const QUrl &document, const QObject *context, const SwitchSourceHeaderHandler &h)
 {
     return d->clangdSwitchSourceHeader(document, make_handler(h, context, parseClangdSwitchSourceHeader));
@@ -1737,6 +1839,12 @@ LSPClientServer::RequestHandle LSPClientServer::clangdMemoryUsage(const QObject
     return d->clangdMemoryUsage(make_handler(h, context, identity));
 }
 
+LSPClientServer::RequestHandle
+LSPClientServer::rustAnalyzerExpandMacro(const QObject *context, const QUrl &document, const LSPPosition &pos, const ExpandMacroHandler &h)
+{
+    return d->rustAnalyzerExpandMacro(document, pos, make_handler(h, context, parseExpandedMacro));
+}
+
 LSPClientServer::RequestHandle
 LSPClientServer::documentFormatting(const QUrl &document, const LSPFormattingOptions &options, const QObject *context, const FormattingReplyHandler &h)
 {
